#ifndef __SA1100_H__
#define __SA1100_H__


/***********************************************
  Interrupt Setup Functions
***********************************************/

void SetIrqLevel( unsigned uIrq, unsigned isFIQ );
void EnableIrq( unsigned uMask );
void DisableIRQ( unsigned uMask );
void DisableAllInts( void );
unsigned Install_Handler( unsigned routine, unsigned *vector );


/***********************************************
  Timer Interrupt Functions
***********************************************/

void EnableTimerIRQ( unsigned timerSel );
void DisableTimerIRQ( unsigned timerSel );
void initTimer( unsigned timerSel, unsigned us );


/***********************************************
  Real Time Clock Functions
***********************************************/

void startRTC( void );
void setRTCdivider( unsigned divider );
unsigned getRTC( void );
void setRTC( unsigned val );


/***********************************************
  Busy Wait Function
***********************************************/

void Wait( unsigned uSec );


/***********************************************
  MMU Functions
***********************************************/

void CleanDcache( void );
void CleanAllCaches( void );


/***********************************************
  Hex LED Functions
***********************************************/

void _saSetHexLEDs( char );


/***********************************************
  Memory Mapping Constrants/Structs, etc.
***********************************************/

#define BYTES_PER_PIXEL 2
#define FIELDS_PER_FRAME 2

typedef unsigned char UCHAR;
typedef int WORD;
typedef unsigned long int DWORD;
typedef unsigned short PIXEL;

#ifdef FALSE
#  undef FALSE
#endif

#ifdef TRUE
#  undef TRUE
#endif

#define FALSE	0
#define TRUE	1

#define IRQ_VECTOR 0x18
#define FIQ_VECTOR 0x1C

#ifdef TIMERTICK
#undef TIMERTICK
#endif

#ifdef TIMER_SECOND
#undef TIMER_SECOND
#endif

#define   TIMERTICK   4             // 1 microsecond is 3.7 clock ticks 
#define   TIMER_SECOND (1000 * 1000)

#define EXT(d, b) ((((unsigned int)(d))>>(b))&1)

/*           XBOX GPIO Control Registers               */
#define XGP_BASE    0x100A0000 /* XBOX GP Base         */
#define XGP_SET_REG 0x100A0000 /* XBOX GP Set Reg      */
#define XGP_RD_CNT  0x100A0004 /* XBOX GP Count to Set */
#define XGP_RD_SET  0x100A0008 /* XBOX GP Read Set     */
#define XGP_RD_GP   0x100A000C /* XBOX Read Current GP */
#define XGP_SET_PUL 0x100A0010 /* XBOX Set GP Pulse    */ 

/*   System Control Module GPIO Register Locations     */
#define GP_BASE     0x90040000 /*  SCM GP BASE         */
#define GP_PIN_LVL  0x90040000 /*  Pin Level           */
#define GP_PIN_DIR  0x90040004 /*  Direction           */
#define GP_PIN_SET  0x90040008 /*  Output Set          */
#define GP_PIN_CLR  0x9004000C /*  Output Clr          */
#define GP_PIN_RED  0x90040010 /*  Rising Edge Detect  */
#define GP_PIN_FED  0x90040014 /*  Falling Edge Detect */
#define GP_PIN_EDS  0x90040018 /*  Edge Detect Status  */
#define GP_PIN_AFR  0x9004001C /*  Alternate Function  */

/* Power Manager Register locations */
#define PM_BASE     0x90020000 /*  Power Manager Register Base */

#define TUCR		0x90030008
#define TUCR_TIC	1<<6
#define TUCR_TTST	1<<7
#define TUCR_RCRC	1<<8
#define TUCR_PMD	1<<9
#define TUCR_MR		1<<10
#define TUCR_CTB0	1<<20
#define TUCR_CTB1	1<<21
#define TUCR_CTB2	1<<22
#define TUCR_FDC	1<<23
#define TUCR_FMC	1<<24
#define TUCR_TMC	1<<25
#define TUCR_DPS	1<<26
#define TUCR_3_POINT_68_Mhz	1<<29

#define MC_BASE 0xA0000000	/* Memory and PMCICA controller register base */ 
# define SA_SRR 0x18800000	/* SideARM System Reset Register */
# define MCP_RESET_BIT (1<<3)	/* bit to reset UCB-1200 */

/*  System Control Module IC Register Locations     */
#define IC_BASE     0x90050000 /*  SCM IC Base      */
#define IC_PEND     0x90050000 /*  Pending          */
#define IC_MASK     0x90050004 /*  Mask             */
#define IC_LEVEL    0x90050008 /* Level 0-IRQ 1-FIQ */

/* Set up memory space to store interrupt results   */
#define INT_STRUCT_BASE  0xC0080000 /* icstructbase */

#define CLEAR       0xFFFFFFFF /*  Clear Reg Pattern  */
#define VERBOSE     0          /*  print debug stuff  */

#define CLK_SCALE_TO_59Mhz 	0
#define CLK_SCALE_TO_74Mhz	(CLK_SCALE_TO_59Mhz+1)
#define CLK_SCALE_TO_88Mhz	(CLK_SCALE_TO_74Mhz+1)
#define CLK_SCALE_TO_103Mhz	(CLK_SCALE_TO_88Mhz+1)
#define CLK_SCALE_TO_118Mhz	(CLK_SCALE_TO_103Mhz+1)
#define CLK_SCALE_TO_133Mhz	(CLK_SCALE_TO_118Mhz+1)
#define CLK_SCALE_TO_148Mhz	(CLK_SCALE_TO_133Mhz+1)
#define CLK_SCALE_TO_162Mhz	(CLK_SCALE_TO_148Mhz+1)
#define CLK_SCALE_TO_178Mhz	(CLK_SCALE_TO_162Mhz+1)
#define CLK_SCALE_TO_192Mhz	(CLK_SCALE_TO_178Mhz+1)
#define CLK_SCALE_TO_206Mhz	(CLK_SCALE_TO_192Mhz+1)
#define CLK_SCALE_TO_220Mhz	(CLK_SCALE_TO_206Mhz+1)

#define BOUNCE_INC 22
#define STATUS_LINE_SIZE 16

#define GPIO_00			(1 		<< 0)
#define GPIO_01			(GPIO_00 	<< 1)
#define GPIO_02			(GPIO_01 	<< 1)
#define GPIO_03			(GPIO_02	<< 1)
#define GPIO_04			(GPIO_03 	<< 1)
#define GPIO_05			(GPIO_04 	<< 1)
#define GPIO_06			(GPIO_05 	<< 1)
#define GPIO_07			(GPIO_06 	<< 1)
#define GPIO_08			(GPIO_07 	<< 1)
#define GPIO_09			(GPIO_08 	<< 1)
#define GPIO_0A			(GPIO_09 	<< 1)
#define GPIO_0B			(GPIO_0A 	<< 1)
#define GPIO_0C			(GPIO_0B 	<< 1)
#define GPIO_0D			(GPIO_0C 	<< 1)
#define GPIO_0E			(GPIO_0D 	<< 1)
#define GPIO_0F			(GPIO_0E 	<< 1)
#define GPIO_10			(GPIO_0F 	<< 1)
#define GPIO_11			(GPIO_10 	<< 1)
#define GPIO_12			(GPIO_11 	<< 1)
#define GPIO_13			(GPIO_12 	<< 1)
#define GPIO_14			(GPIO_13 	<< 1)
#define GPIO_15			(GPIO_14 	<< 1)
#define GPIO_16			(GPIO_15 	<< 1)
#define GPIO_17			(GPIO_16 	<< 1)
#define GPIO_18			(GPIO_17 	<< 1)
#define GPIO_19			(GPIO_18 	<< 1)
#define GPIO_1A			(GPIO_19 	<< 1)
#define GPIO_1B			(GPIO_1A 	<< 1)


#define IRQ_GPIO_00		(1 			<< 0)
#define IRQ_GPIO_01		(IRQ_GPIO_00 		<< 1)
#define IRQ_GPIO_02		(IRQ_GPIO_01 		<< 1)
#define IRQ_GPIO_03		(IRQ_GPIO_02 		<< 1)
#define IRQ_GPIO_04		(IRQ_GPIO_03 		<< 1)
#define IRQ_GPIO_05		(IRQ_GPIO_04 		<< 1)
#define IRQ_GPIO_06		(IRQ_GPIO_05 		<< 1)
#define IRQ_GPIO_07		(IRQ_GPIO_06 		<< 1)
#define IRQ_GPIO_08		(IRQ_GPIO_07 		<< 1)
#define IRQ_GPIO_09		(IRQ_GPIO_08 		<< 1)
#define IRQ_GPIO_10		(IRQ_GPIO_09 		<< 1)
#define IRQ_GPIO_11_GPIO_27	(IRQ_GPIO_10 		<< 1)
#define IRQ_LCD_SERVICE_REQ	(IRQ_GPIO_11_GPIO_27 	<< 1)
#define IRQ_UDC_SERVICE_REQ	(IRQ_LCD_SERVICE_REQ 	<< 1)
#define IRQ_SDLC		(IRQ_UDC_SERVICE_REQ 	<< 1)
#define IRQ_UART0		(IRQ_SDLC		<< 1)
#define IRQ_UART1		(IRQ_UART0 		<< 1)
#define IRQ_UART2		(IRQ_UART1 		<< 1)
#define IRQ_MCP			(IRQ_UART2 		<< 1)
#define IRQ_SSP			(IRQ_MCP 		<< 1)
#define IRQ_CHANNEL0		(IRQ_SSP 		<< 1)
#define IRQ_CHANNEL1		(IRQ_CHANNEL0 		<< 1)
#define IRQ_CHANNEL2		(IRQ_CHANNEL1 		<< 1)
#define IRQ_CHANNEL3		(IRQ_CHANNEL2 		<< 1)
#define IRQ_CHANNEL4		(IRQ_CHANNEL3 		<< 1)
#define IRQ_CHANNEL5		(IRQ_CHANNEL4 		<< 1)
#define IRQ_OSTIMER0		(IRQ_CHANNEL5 		<< 1)
#define IRQ_OSTIMER1		(IRQ_OSTIMER0 		<< 1)
#define IRQ_OSTIMER2		(IRQ_OSTIMER1 		<< 1)
#define IRQ_OSTIMER3		(IRQ_OSTIMER2 		<< 1)
#define IRQ_ONE_HZ_CLK		(IRQ_OSTIMER3 		<< 1)
#define IRQ_RTC_ALARM		(IRQ_ONE_HZ_CLK		<< 1)

#define GPIO_LOWEST_OR  11                    /* GP Lowest OR                   */
#define GPIO_INTOR_MASK (1<<GPIO_LOWEST_OR)   /* GP[27:GPIOLOWESTOR] Int Number */ 
#define GPIO_INT_MASK \
            (0xFFFFFFFF>>(31-GPIO_LOWEST_OR)) /* GP Interrupt Mask  */

#define RESET_EXCEPTION 		(0x00)
#define UNDEFINED_INSTRUCTION_EXCEPTION (RESET_EXCEPTION +0x04)
#define SWI_EXCEPTION_EXCEPTION 	(UNDEFINED_INSTRUCTION_EXCEPTION +0x04)
#define PREFETCH_ABORT_EXCEPTION 	(SWI_EXCEPTION_EXCEPTION +0x04)
#define DATA_ABORT_EXCEPTION 		(PREFETCH_ABORT_EXCEPTION +0x04)
#define UNUSED_VECTOR 			(DATA_ABORT_EXCEPTION +0x04)
#define IRQ_EXCEPTION 			(UNUSED_VECTOR +0x04)
#define FIQ_EXCEPTION 			(IRQ_EXCEPTION +0x04)

#define IRQ         0x1
#define FIQ         0x2

struct int_struct {
  int ic_pending1;    // RW
  int ic_pending2;    // RW
  int qdone;          // RW
  int gp_reset_state; // RW
  int gp_state;       // RW
};

struct xbox_gp_regs {
  int xgpsr;        // W
  int xgprdcnt;     // R
  int xgprdset;     // R
  int xgprdgp;      // R
  int xgpsetpul;    // W
};

/* GDS Power manager Registers */
struct pm_gp_regs {
  int pmcr;  // 
  int pssr;    // 
  int pspr;    // 
  int pwer;    // 
  int pcrf;    //  
  int ppcr;    // 
  int pgsr;    // 
  int posr;   // 
};

/* Memory and PCMCIA Control Registers */
struct mc_gp_regs {
  int mdcnfg;  // 
  int mdcas0;    // 
  int mdcas1;    // 
  int mdcas2;    // 
  int msc0;    //  
  int msc1;    // 
  int mecr;    // 
};

struct scm_gpio_regs {
  int gplr;         // R : pin level register
  int gpdr;    	    // RW: pin direction register
  int gpsr;         // W : pin set register
  int gpcr;         // W : pin clear register
  int grer;         // RW: rising edge register
  int gfer;         // RW: falling edge register
  int gedr;         // RW: edge detect status register
  int gafr;         // RW: alternate function register
};

#define ICLR_IRQ 0
#define ICLR_FIQ 1
struct scm_ic_regs {
  int icip; 		// R
  int icmr;		// RW
  int iclr;		// RW
  int reserved0;        // --
  int icfp;		// R
  int reserved1;        // --
  int reserved2;        // --
  int reserved3;        // --
  int icpr;		// R
};

struct sspreg {
	int sscr0;
	int sscr1;
	int ssppad0;
	int ssdr;
	int ssppad1;
	int sssr;
};

/* SSP CR0 */
#define SSPCR0_M_DSS 0x0000000F
#define SSPCR0_M_FRF 0x00000030
#define SSPCR0_M_SSE 0x00000080
#define SSPCR0_M_SCR 0x0000FF00
#define SSPCR0_V_DSS 0
#define SSPCR0_V_FRF 4
#define SSPCR0_V_SSE 7
#define SSPCR0_V_SCR 8

/* SSP CR1 */
#define SSPCR1_M_RIM 0x00000001
#define SSPCR1_M_TIN 0x00000002
#define SSPCR1_M_LBM 0x00000004
#define SSPCR1_V_RIM 0
#define SSPCR1_V_TIN 1
#define SSPCR1_V_LBM 2

/* SSP status */
#define SSPSR_M_TNF 0x00000002
#define SSPSR_M_RNE 0x00000004
#define SSPSR_M_BSY 0x00000008
#define SSPSR_M_TFS 0x00000010
#define SSPSR_M_RFS 0x00000020
#define SSPSR_M_ROR 0x00000040
#define SSPSR_V_TNF 1
#define SSPSR_V_RNE 2
#define SSPSR_V_BSY 3
#define SSPSR_V_TFS 4
#define SSPSR_V_RFS 5
#define SSPSR_V_ROR 6

#define SSP_BASE 0x80070060
#define CONTROL_BITS		(0x70)
#define MMU_ENABLE 		(1 << 0)
#define ADDRESS_FAULT_ENABLE 	(1 << 1)
#define D_CACHE_ENABLE 		(1 << 2)
#define WBUFF_ENABLE 		(1 << 3)
#define BIG_ENDIAN 		(1 << 7)
#define SYSTEM_CHECKS 		(1 << 7)
#define ROM_CHECKS 		(1 << 9)
#define I_CACHE_ENABLE 		(1 << 12)
#define I_VECTOR_ALIGN 		(1 << 13)

struct os_timer_regs {
  int osmr[4];			// r/w
  int oscr;			// r/w
  int ossr;			// r
  int ower;
  int oier;
};
/* 		OS Timer Registers                      */
#define OSMR_0	    0x90000000 /* OS Timer Match Registers 0  */
#define OSMR_1	    0x90000004 /* OS Timer Match Registers 1  */
#define OSMR_2	    0x90000008 /* OS Timer Match Registers 2  */
#define OSMR_3	    0x9000000C /* OS Timer Match Registers 3  */

#define OSCR	    0x90000010 /* OS Timer Counter Registers   */
#define OSSR	    0x90000014 /* OS Timer Status  Registers   */
#define OWER	    0x90000018 /* OS Timer Watchdog Registers  */
#define OIER	    0x9000001C /* OS Timer Interrupt enable Registers */

/* 		OS Timer Registers                      */
/* These are ordered according to the tests in Ashleys area */
/* They DO NOT match the current V2.0 spec as of 4-19-97    */
struct lcd_regs {
  int Control0;     /* LCCR0 */
  int Status;       /* LCSR  */
  int Reserved1;    /* Pad   */
  int Reserved2;    /* Pad   */
  int DMA1Base;     /* DBAR1 */
  int DMA1Current;  /* DCAR1 */
  int DMA2Base;     /* DBAR2 */
  int DMA2Current;  /* DCAR2 */
  int Control1;     /* LCCR1 */
  int Control2;     /* LCCR2 */
  int Control3;     /* LCCR3 */
};

#define SP3R_BASE (0x80050000)

struct sp3_regs {
  unsigned utcr0;	/* UART Control Register 0 */
  unsigned utcr1;	/* UART Control Register 1 */
  unsigned utcr2;	/* UART Control Register 2 */
  unsigned utcr3;	/* UART Control Register 3 */
  unsigned res0;       /* reserved */
  unsigned utdr;	/* UART Data    Register */
  unsigned res1;	/* reserved */
  unsigned utsr0;	/* UART Status Register 0 */
  unsigned utsr1;	/* UART Status Register 1 */
};


#define ICPR_BASE (0x80030000)
#define ICP_CR0_PE	(1<<0)
#define ICP_CR0_OES	(1<<1)
#define ICP_CR0_SBS	(1<<2)
#define ICP_CR0_DSS	(1<<3)
#define ICP_CR0_SCE	(1<<4)
#define ICP_CR0_RCE	(1<<5)
#define ICP_CR0_TCE	(1<<6)

#define ICP_CR3_RXE	(1<<0)
#define ICP_CR3_TXE	(1<<1)
#define ICP_CR3_BRK	(1<<2)
#define ICP_CR3_RIM	(1<<3)
#define ICP_CR3_TIM	(1<<4)
#define ICP_CR3_LBM	(1<<5)

#define ICP_SR0_TFS	(1<<0)
#define ICP_SR0_RFS	(1<<1)
#define ICP_SR0_RID	(1<<2)
#define ICP_SR0_RBB	(1<<3)
#define ICP_SR0_REB	(1<<4)
#define ICP_SR0_RIF	(1<<5)

#define ICP_SR1_TBY	(1<<0)
#define ICP_SR1_RNE	(1<<1)
#define ICP_SR1_TNF	(1<<2)
#define ICP_SR1_PRE	(1<<3)
#define ICP_SR1_ROR	(1<<4)

#define ICP_CR4_HSE	(1<<0)
#define ICP_CR4_LPM	(1<<1)


struct icp_regs {
  unsigned utcr0;	/* UART Control Register 0 */
  unsigned utcr1;	/* UART Control Register 1 */
  unsigned utcr2;	/* UART Control Register 2 */
  unsigned utcr3;	/* UART Control Register 3 */
  unsigned utcr4;	/* UART Control Register 4 */
  unsigned utdr;	/* UART Data    Register */
  unsigned res0;	/* reserved */
  unsigned utsr0;	/* UART Status Register 0 */
  unsigned utsr1;	/* UART Status Register 1 */
};

#define HSSP_HSCR0_ITR	(1<<0)
#define HSSP_HSCR0_LBM	(1<<1)
#define HSSP_HSCR0_TUS	(1<<2)
#define HSSP_HSCR0_TXE	(1<<3)
#define HSSP_HSCR0_RXE	(1<<4)
#define HSSP_HSCR0_RIM	(1<<5)
#define HSSP_HSCR0_TIM	(1<<6)
#define HSSP_HSCR0_AME	(1<<7)


#define HSSP_BASE (0x80020060)
struct hssp_regs {
  unsigned hscr0;	/* HSSP Control Register 0 */
  unsigned hscr1;	/* HSSP Control Register 1 */
  unsigned res0;	/* reserved */
  unsigned hsdr;	/* HSSP Data Register */
  unsigned res1;	/* reserved */
  unsigned hssr0;	/* HSSP Status Register 0 */
  unsigned hssr1;	/* HSSP Status Register 1 */
};

// Peripheral Pin Controller registers
#define PPC_BASE (0x90060000)
#define PPDR_LDD	(0xFF<<0)
#define PPDR_LP_CLK	(1<<8)
#define PPDR_LL_CLK	(1<<9)
#define PPDR_LF_CLK	(1<<10)
#define PPDR_L_BIAS	(1<<11)
#define PPDR_TXD0	(1<<12)
#define PPDR_RXD0	(1<<13)
#define PPDR_TXD1	(1<<14)
#define PPDR_RXD1	(1<<15)
#define PPDR_TXD2	(1<<16)
#define PPDR_RXD2	(1<<17)
#define PPDR_TXD3	(1<<18)
#define PPDR_RXD3	(1<<19)
#define PPDR_TXD4	(1<<20)
#define PPDR_RXD4	(1<<21)
#define PPDR_SCLK	(1<<22)
#define PPDR_SFRM	(1<<23)


struct ppc_regs {
  unsigned ppdr;	/* PPC Pin Direction Register */
  unsigned ppsr;	/* PPC Pin State Register */
  unsigned ppar;	/* PPC Pin Assignment Register */
  unsigned psdr;	/* PPC Sleep Mode Direction Register */
  unsigned ppfr;	/* PPC Pin Flag Register */
  int dummy1;
  int dummy2;
  int dummy3;
  int dummy4;
  int dummy5;
  int hscr2;
  int dummy6;
  int mccr1;
};

/* UCB-1200 */
#define MCPBASE 0x80060000

#define MCCR_V_ASD 0
#define MCCR_V_TSD 8
#define MCCR_M_MCE 0x010000
#define MCCR_M_ECS 0x020000
#define MCCR_M_ADM 0x040000
#define MCCR_M_TTM 0x080000
#define MCCR_M_TRM 0x100000
#define MCCR_M_ATM 0x200000
#define MCCR_M_ARM 0x400000
#define MCCR_M_LBM 0x800000

#define MCDR2_V_RN 17
#define MCDR2_M_nRW 0x10000

#define MCSR_M_ATS 0x00000001
#define MCSR_M_ARS 0x00000002
#define MCSR_M_TTS 0x00000004
#define MCSR_M_TRS 0x00000008
#define MCSR_M_ATU 0x00000010
#define MCSR_M_ARO 0x00000020
#define MCSR_M_TTU 0x00000040
#define MCSR_M_TRO 0x00000080
#define MCSR_M_ANF 0x00000100
#define MCSR_M_ANE 0x00000200
#define MCSR_M_TNF 0x00000400
#define MCSR_M_TNE 0x00000800
#define MCSR_M_CWC 0x00001000
#define MCSR_M_CRC 0x00002000
#define MCSR_M_ACE 0x00004000
#define MCSR_M_TCE 0x00008000

struct mcpreg {
	int mccr;
	int rsv0;
	int mcdr0;
	int mcdr1;
	int mcdr2;
	int rsv1;
	int mcsr;
};

#define RTCBASE (0x90010000)
struct scm_rtc_Regs {
	int rtar;
	int rcnr;
	int rttr;
	int dummy;
	int rtsr;
};

// added:JWN
#define KPIOR		0x18800004
#define DLEDR		0x18800006

#endif /* __SA1100_H__ */