;
; sa1100.s
;
; Copyright  1999  University of Washington
;
; Portions Copyright  1998 Intel Corp.
;
;
; This is an amalgamation of several assembly files gleaned from various
; examples and library source.  It provides many basic functions which 
; must be setup at the assembly level.
;
; See the function documentations for information specific to each function.
;
;
; Revision History:
;
;    Created Sept 30, 1999 - Jeff Nichols
;
	EXPORT  CleanDcache
        EXPORT  CleanAllCaches
  	EXPORT  _saFastMemCopy
	EXPORT  _saSetHexLEDs
	EXPORT  clearCPSRBits
	EXPORT  getCPSRBits
	EXPORT  WriteCP15
	EXPORT  ReadCP15
	EXPORT  _saMemReadBenchmark
	EXPORT  _saMemWriteBenchmark
	KEEP


;*****************************************************************************
;*                              app constants                                *
;*****************************************************************************

SIXTEEN_BIT_MODE	EQU 1
FAST_FIND_SYNC		EQU 1
SYSREG_B		EQU 0x18c00000


;*****************************************************************************
;*                              MMU constants                                *
;*****************************************************************************

MMUCP                   CP      15

MMUIDReg                CN      0
MMUControlReg           CN      1
MMUPageTableBaseReg     CN      2
MMUDomainAccessReg      CN      3
MMUFaultStatusReg       CN      5
MMUFaultAddressReg      CN      6
MMUFlushIDCReg          CN      7
MMUFlushTLBReg          CN      8
MMUTLBReg		CN	2
MMUReadBufferReg	CN	9

PageTable1Size          EQU     4 * 4 * 1024 ; one per megabyte

MMU                     EQU     1 ; 0 is off, 1 is on
AlignFault              EQU     0 ; 0 is off, 2 is on
Cache                   EQU     4 ; 0 is off, 4 is on
WriteBuffer             EQU     8 ; 0 is off, 8 is on
Config26Bit             EQU     0 ; mmu, idc, wb off, 26 bit prog/data, little endian
Config32Bit             EQU     &10 + &20 ; mmu, idc, wb off, 32 bit prog/data, little endian
EnableMMU26             EQU     Config26Bit + MMU + AlignFault + Cache + WriteBuffer
EnableMMU32             EQU     Config32Bit + MMU + AlignFault + Cache + WriteBuffer

;       Read Buffer Coprocessor Operations
RB_FLUSH_ALL 		EQU 		0
RB_FLUSH_DISCRETE	EQU		1
RB_LOAD  		EQU		2
;       Cache Operations
CP15_FLUSH_ALL		EQU   0
CP15_FLUSH_D		EQU   0
CP15_SINGLE_ENTRY_OP    EQU   1
CP15_DRAIN_WRITE_BUFFER EQU   4

Mode_USR        EQU     0x10
Mode_FIQ	EQU	0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_SYS	EQU	0x1F

I_Bit           EQU     0x80
F_Bit           EQU     0x40

; Locations of various things in our memory system

PHYSICAL_RAM_START  	EQU     0xC0000000
VIRTUAL_RAM_START	EQU 	0x00000000

STACK_SIZE		EQU     1024


;*****************************************************************************
;*                              mmu functions                                *
;*****************************************************************************

					AREA mmu, CODE
CleanDcache
	STMFD r13!,{r0-r1,lr}        	; store registers
	mov r0,#&e0000000
	add r1,r0,#8192
loop
	ldr r2,[r0],#32
	teq r1,r0
	bne loop
	MCR MMUCP,0,r0,MMUFlushIDCReg,c6,CP15_FLUSH_D	; Flush Data
	NOP
	NOP
	NOP
	LDMFD r13!,{r0-r1,lr}         	; Restore original registers
	mov pc,lr

CleanAllCaches
	STMFD r13!,{r0-r1,lr}        	; store registers
	mov r0,#&e0000000
	add r1,r0,#8192
loop2
	ldr r2,[r0],#32
	teq r1,r0
	bne loop2
	MCR MMUCP,0,r0,MMUFlushTLBReg,c7,CP15_FLUSH_ALL ; Flush the TLB
        NOP
        NOP
        NOP
	MCR MMUCP,0,r0,MMUFlushIDCReg,c7,CP15_FLUSH_ALL	; Flush Data
	NOP
	NOP
	NOP
	LDMFD r13!,{r0-r1,lr}         	; Restore original registers
	mov pc,lr


	AREA PageTables,DATA,ALIGN=14
	OPT	 32+128+512+8192+1

; Access Permissions - not shifted into position.
NO_ACCESS  * 0  ; Depending on the setup, 0 represents one of 
SVC_R      * 0  ; these two access permissions (see ARM600 datasheet)
SVC_RW     * 1
NO_USR_W   * 2
ALL_ACCESS * 3
; U, C and B bits in their correct positions (see ARM600 datasheet)
U_BIT * 16
C_BIT  * 8
B_BIT * 4

; Entry type
SECTION    * 2
PAGE       * 1
INVALID    * 0

SMALL      * 2
LARGE      * 1

    MACRO
$L  L1Entry $type, $addr, $dom, $ucb, $acc
$L  IF ($type=SECTION)
      DCD ((($addr):AND:&FFF00000):OR:(($acc):SHL:10) \
          :OR:(($dom):SHL:5):OR:($ucb):OR:($type))
      MEXIT
    ENDIF
    IF ($type=PAGE)
      DCD ((($addr):AND:&FFFFFC00):OR:(($dom):SHL:5) \
          :OR:(($ucb):AND:U_BIT):OR:$type)
    ELSE
      DCD 0 ; Invalid Level 1 Page Table Entry
    ENDIF
    MEND

    GBLA    counter
PTables
;;
; Page tables
;;
;; this is the dram/rom table
dram_rom
    L1Entry SECTION, (0xc00:SHL:20), 0,C_BIT | B_BIT, ALL_ACCESS; DRAM0,0 
    L1Entry SECTION, (0xc01:SHL:20), 0,C_BIT | B_BIT, ALL_ACCESS; DRAM0,1 
    L1Entry SECTION, (0xc02:SHL:20), 0,C_BIT | B_BIT, ALL_ACCESS; DRAM0,2
    L1Entry SECTION, (0xc03:SHL:20), 0,C_BIT | B_BIT, ALL_ACCESS; DRAM0,3
    L1Entry SECTION, (0xc04:SHL:20), 0,C_BIT | B_BIT, ALL_ACCESS; DRAM1,0 
    L1Entry SECTION, (0xc05:SHL:20), 0,B_BIT, ALL_ACCESS; DRAM1,1 
    L1Entry SECTION, (0xc06:SHL:20), 0,B_BIT, ALL_ACCESS; DRAM1,2 
    L1Entry SECTION, (0xc07:SHL:20), 0,B_BIT, ALL_ACCESS; DRAM1,3 
    L1Entry SECTION, (0xc08:SHL:20), 0,B_BIT, ALL_ACCESS; DRAM2,0 
    L1Entry SECTION, (0xc09:SHL:20), 0,B_BIT, ALL_ACCESS; DRAM2,1 
    L1Entry SECTION, (0xc0A:SHL:20), 0,B_BIT, ALL_ACCESS; DRAM2,2 
    L1Entry SECTION, (0xc0B:SHL:20), 0,B_BIT, ALL_ACCESS; DRAM3,3 
    L1Entry SECTION, (0xc0C:SHL:20), 0,B_BIT, ALL_ACCESS; DRAM3,0 
    L1Entry SECTION, (0xc0D:SHL:20), 0,B_BIT, ALL_ACCESS; DRAM3,1 
    L1Entry SECTION, (0xc0E:SHL:20), 0,C_BIT | B_BIT, ALL_ACCESS; DRAM3,2 
    L1Entry SECTION, (0xC0F:SHL:20), 0,C_BIT | B_BIT, ALL_ACCESS; DRAM3,3 
    L1Entry SECTION, (0xc80:SHL:20), 0,B_BIT, ALL_ACCESS;  
    L1Entry SECTION, (0xc81:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc82:SHL:20), 0,B_BIT, ALL_ACCESS;
    L1Entry SECTION, (0xc83:SHL:20), 0,B_BIT, ALL_ACCESS;
    L1Entry SECTION, (0xc84:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc85:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc86:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc87:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc88:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc89:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc8A:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc8B:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc8C:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc8D:SHL:20), 0,B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xc8E:SHL:20), 0,C_BIT | B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0xC8F:SHL:20), 0,C_BIT | B_BIT, ALL_ACCESS; 
    L1Entry SECTION, (0:SHL:20),     0,B_BIT, ALL_ACCESS; ROM 
 
counter SETA 33
    WHILE   counter < 0xc00 ; map the area upto E000 0000 space as memory 1-1 for rest 
    L1Entry SECTION, (counter:SHL:20), 0,0, ALL_ACCESS
counter SETA counter+1
    WEND

    L1Entry SECTION, (counter:SHL:20), 0,B_BIT, ALL_ACCESS
counter SETA counter+1

    WHILE   counter < 0xe00 ; map the area upto E000 0000 space as memory 1-1 for rest 
    L1Entry SECTION, (counter:SHL:20), 0,0, ALL_ACCESS
counter SETA counter+1
    WEND

    L1Entry SECTION, (counter:SHL:20), 0,C_BIT | B_BIT, ALL_ACCESS  ; Cache clean space
counter SETA counter+1
    L1Entry SECTION, (counter:SHL:20), 0,C_BIT, ALL_ACCESS			; More ZEROS
counter SETA counter+1

    WHILE   counter < 0x1000 ;  all the rest as I/O 
    L1Entry SECTION, (counter:SHL:20), 0,B_BIT, ALL_ACCESS	
counter SETA counter+1
    WEND


;*****************************************************************************
;*                          hex led functions                                *
;*****************************************************************************

    MACRO 
	READCP $reg
	IF ($reg=4:LOR:$reg=7:LOR:$reg=8:LOR:$reg=9:LOR:$reg=10:LOR:$reg=11:LOR:$reg=12)
          MOVEQ r0,#&FFFFFFFF	; Reserved register
	ELSE
          MRCEQ p15,0,r0,c$reg,c0
 	ENDIF
        MOVEQ pc,lr
	SUBS r0,r0,#1
    MEND

   
    MACRO 
	WRITECP $reg
	IF ($reg=0:LOR:$reg=4:LOR:$reg=10:LOR:$reg=11:LOR:$reg=12)
          MOVEQ r0,#&FFFFFFFF	; Reserved register
	ELSE
	   MCREQ p15,0,R1,c$reg,c0,0	; Write to Control Register 
 	ENDIF
        MOV   r0,r0
        MOV   r0,r0
        MOV   r0,r0
        MOVEQ pc,lr		; return from subroutine
	SUBS r0,r0,#1
     MEND

  MACRO
       NOP
        MOV r0,r0
  MEND

  MACRO
        C_dumpDWtoHEXLeds $a,$b,$c
        LDR r0,=$c
        LDR r1,=$b
        LDR r2,=$a
  IMPORT  dumpDWtoHEXLeds
        bl dumpDWtoHEXLeds
  MEND

  MACRO
        CLOCK_FIFO $DstReg,$AddrReg
  IF (SIXTEEN_BIT_MODE=1)
	LDRH  r2,[$AddrReg],#4			; r2  = 0   0   a   b
	LDRH  $DstReg,[$AddrReg],#4	        ; dst = 0   0   c   d
	ORR   $DstReg,r2,$DstReg,LSL #16	; dst = c   d   b   a
  ELSE
	LDR   $DstReg,[$AddrReg],#4
  ENDIF

    MEND

    MACRO
      MSET $addr,$val,$regA,$regB
	LDR  $regA,=$addr
	LDR  $regB,=$val
	STR  $regB,[$regA]
    MEND

    MACRO
      MGET $reg,$addr
	LDR $reg,=$addr	; get address of Fifo
	LDR $reg,[$reg]		
    MEND

    MACRO
     GPIO_ON
     ldr r0,=0x90040000
     ldr r1,[r0,#0]
     ldr r0,=0x9004000c
     orr r1,r1,#&100000			; bit position selects which GPIO
     str r1,[r0]
    MEND

    MACRO
     GPIO_OFF
     ldr r0,=0x90040000
     ldr r1,[r0,#0]
     ldr r0,=0x90040008
     orr r1,r1,#&100000			; bit position selects which GPIO
     str r1,[r0]
    MEND
; ------------------------------------------------------------------
	; macro to set the Hex LEDs to the value given in $w1
	; Variant which writes to numeric display as well as LEDs.

	MACRO
$label 	NSET_LEDS	$w1, $w2, $w3		;; this macro changed for MESA
	; Write to numeric display
	LDR	$w3, =SYSREG_B
	MOV	$w2, $w1
	AND	$w2,$w2,#0xF	;
	; Display First 4 bits on the Left Leds
	ORR	$w2, $w2, #0x30
	STRH	$w2, [$w3]
	AND	$w2, $w2, #0xEF
	STRH	$w2, [$w3]
	ORR	$w2, $w2, #0x30
	STRH	$w2, [$w3]
	; Display Second 4 bits on the rigth Leds
	MOV	$w2, $w1, LSR #4
	AND	$w2, $w2, #0xF	;
	ORR	$w2, $w2, #0x30
	STRH	$w2, [$w3]
	AND	$w2, $w2, #0xDF
	STRH	$w2, [$w3]
	ORR	$w2, $w2, #0x30
	STRH	$w2, [$w3]
	SET_LEDS	$w1, $w2, $w3
	MEND

  AREA |OBC$$code|, CODE, READONLY

BYTES_PER_BURST8		EQU 32
BYTES_PER_BURST4		EQU 16
BYTES_PER_BURST2		EQU  8
BYTES_PER_WORD			EQU  4

_saFastMemCopy
  STMFD sp!,{r3-r10,lr}      ; store registers
TestBurst8  
  cmp    a3,#BYTES_PER_BURST8
  blt    TestBurst4

  MCR    p15,0,a2,MMUReadBufferReg,c9,RB_LOAD  ; Read buffer 1 with Burst of 8
  LDMIA  a2!, {r3-r10}
  STMIA  a1!,{r3-r10}      ; store registers

  subs   a3,a3,#BYTES_PER_BURST8
  bne    TestBurst8	; not found ? keep testing

TestBurst4
  cmp    a3,#BYTES_PER_BURST4
  blt    TestBurst1

  MCR    p15,0,a2,MMUReadBufferReg,c5,RB_LOAD  ; Read buffer 1 with Burst of 4
  LDMIA  a2!, {r3-r6}
  STMIA  a1!,{r3-r6}      ; store registers

  subs   a3,a3,#BYTES_PER_BURST4
  bne    TestBurst4

TestBurst1
  cmp    a3,#BYTES_PER_WORD
  blt    ExitFastCopy

  LDR    r3,[a2],#BYTES_PER_WORD
  STR    r3,[a1],#BYTES_PER_WORD

  subs   a3,a3,#BYTES_PER_WORD
  bne  	 TestBurst1
  
ExitFastCopy
  LDMFD sp!,{r3-r10,pc}      ; Restore original registers

_saMemReadBenchmark
  STMFD sp!,{r3-r6,lr}      ; store registers
  mov r3,#0xd8000000
  mov r4,#0xd9000000
  mov r5,#0xda000000
  mov r6,#0xdb000000

BulkBurstRead  
  MCR    p15,0,r3,MMUReadBufferReg,c8,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r4,MMUReadBufferReg,c9,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r5,MMUReadBufferReg,c10,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r6,MMUReadBufferReg,c11,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r3,MMUReadBufferReg,c8,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r4,MMUReadBufferReg,c9,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r5,MMUReadBufferReg,c10,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r6,MMUReadBufferReg,c11,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r3,MMUReadBufferReg,c8,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r4,MMUReadBufferReg,c9,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r5,MMUReadBufferReg,c10,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r6,MMUReadBufferReg,c11,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r3,MMUReadBufferReg,c8,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r4,MMUReadBufferReg,c9,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r5,MMUReadBufferReg,c10,RB_LOAD  ; Read buffer 1 with Burst of 8
  MCR    p15,0,r6,MMUReadBufferReg,c11,RB_LOAD  ; Read buffer 1 with Burst of 8

  subs   a1,a1,#BYTES_PER_BURST8*16
  bne    BulkBurstRead	; not found ? keep testing

  LDMFD sp!,{r3-r6,pc}      ; Restore original registers

_saMemWriteBenchmark
  STMFD sp!,{r3-r10,lr}      ; store registers
  mov   r1,#0x080000
BulkBurstWrite  
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers
  STMIA  r1!,{r3-r10}      ; store registers

  subs   a1,a1,#BYTES_PER_BURST8*16
  bne    BulkBurstWrite	; not found ? keep testing

  LDMFD sp!,{r3-r10,pc}      ; Restore original registers

WriteCP15			; R0 contains data, R1 Value
	SUBS r0,r0,#0

	WRITECP 0		; RESERVED
	WRITECP 1		; Control
	WRITECP 2		; Translation table base
	WRITECP 3		; Domain access control
	WRITECP 4		; RESERVED
	WRITECP 5		; Fault status
	WRITECP 6		; Fault address
	WRITECP 7		; Cache operations
	WRITECP 8		; TLB operations
	WRITECP 9		; Read Buffer operations
	WRITECP 10		; RESERVED
	WRITECP 11		; RESERVED
	WRITECP 12		; RESERVED
	WRITECP 13		; Write Process ID
	WRITECP 14		; Write Breakpoint
	WRITECP 15		; Test, clock, and idle

        MOV r0,#&FFFFFFFF
	MOV   pc,lr		; handle all the rest

ReadCP15
	SUBS r0,r0,#0
				; ReadC0
	READCP 0 	; MMU ID
	READCP 1 	; Control
	READCP 2 	; Translation table base
	READCP 3 	; domain Access Control
	READCP 4	; Reserved
	READCP 5 	; Fault Status
	READCP 6 	; Fault Address
	READCP 7	; RESERVED
	READCP 8	; RESERVED
	READCP 9	; RESERVED
	READCP 10	; RESERVED
	READCP 11	; RESERVED
	READCP 12	; RESERVED
	READCP 13 	; Read Process ID
	READCP 14 	; Read breakpoint
	READCP 15	; RESERVED
	
        MOV r0,#&FFFFFFFF
	MOV   pc,lr		; handle default

_saSetHexLEDs
  STMFD sp!,{r1-r2,lr}      ; store registers
  LDR	r2, =SYSREG_B
  MOV	r1,r0,LSR #4
  AND	r1,r1,#0xF
  AND	r0,r0,#0xF
	             ; Display First 4 bits on the Right LEDs
  ORR	r0,r0,#0x30
  STRH	r0,[r2]
  BIC   r0,r0,#0x20
  STRH	r0,[r2]
  ORR	r0,r0,#0x30
  STRH	r0,[r2]
	            ; Display Second 4 bits on the Left LEDs	
  ORR	r1,r1,#0x30
  STRH	r1,[r2]
  BIC   r1,r1,#0x10
  STRH	r1,[r2]
  ORR	r1,r1,#0x30
  STRH	r1,[r2]

  LDMFD sp!,{r1-r2,pc}      ; Restore original registers

clearCPSRBits
  STMFD sp!,{r1,lr}      ; store registers
  mrs r1,cpsr
  and r1,r1,r0
  msr cpsr,r1
  LDMFD sp!,{r1,pc}      ; Restore original registers

getCPSRBits
  mrs r0,cpsr
  mov pc,lr      ; Restore original registers

                        END
